package edu.iris.dmc.ws.examples;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.List;

import edu.iris.dmc.criteria.CriteriaException;
import edu.iris.dmc.criteria.OutputLevel;
import edu.iris.dmc.criteria.StationCriteria;
import edu.iris.dmc.fdsn.station.model.Channel;
import edu.iris.dmc.fdsn.station.model.Network;
import edu.iris.dmc.fdsn.station.model.Station;
import edu.iris.dmc.service.NoDataFoundException;
import edu.iris.dmc.service.ServiceNotSupportedException;
import edu.iris.dmc.service.ServiceUtil;
import edu.iris.dmc.service.StationService;




public class StationExample {

   public static void main(String[] args) {
	   
      try {
         //initialize the service
         ServiceUtil serviceManager = ServiceUtil.getInstance();
         serviceManager.setAppName("Examples");

         StationService stationService = serviceManager.getStationService();

         // define search criteria
         StationCriteria stationCriteria = new StationCriteria();

         stationCriteria.addNetwork("IU").addNetwork("IM");
         stationCriteria.addStation("A*");
         stationCriteria.addChannel("??Z");
         
         // fetch the data
         List<Network> networks =
         stationService.fetch(stationCriteria, OutputLevel.CHANNEL);
         
         // process the results
         for(Network network:networks){
            System.out.println(network);
   
            // The following loops reflect the StationXML structure
            // Each Network may contain multiple Station(s)
            //   ...which may contain multiple StationEpoch(s)
            //   ...which may contain multiple Channel(s)
            //   ...which may contain multiple ChannelEpoch(s)
   
            for(Station station:network.getStations()){
               System.out.println("\t"+station);
                  for(Channel channel:station.getChannels()){
                     System.out.println("\t\t\t"+channel);
                  }
            }
         }


      } catch (FileNotFoundException e) {
         // No file found
         e.printStackTrace();
      } catch (IOException e) {
         // Something IO didn't work
         e.printStackTrace();
      } catch (NoDataFoundException e) {
         // The query didn't find any data
         e.printStackTrace();
      } catch (CriteriaException e) {
         // Something went wrong with the chosen criteria
         e.printStackTrace();
      } catch (ServiceNotSupportedException e) {
		e.printStackTrace();
	}

   }

}
