package edu.iris.dmc.ws.examples;

import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.TimeZone;

import edu.iris.dmc.criteria.CriteriaException;
import edu.iris.dmc.extensions.entities.Trace;
import edu.iris.dmc.extensions.fetch.TraceData;
import edu.iris.dmc.service.NoDataFoundException;



public class TracesExample {
	public static void main(String[] args) throws IOException, NoDataFoundException, CriteriaException, Exception{
        // define the time range of interest
        DateFormat dfm = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS");
        dfm.setTimeZone(TimeZone.getTimeZone("GMT"));
        Date startDate = dfm.parse("2006-09-11T00:00:00.000");
        Date endDate = dfm.parse("2006-09-11T00:01:45.000");

        //
        // Define the network, station, location and channels to retrieve.
        // 
        String network = "IU";
        String station = "AN*,B*";
        String location = "00";
        String channel = "BHZ";
        Boolean includeResponse = true;
        
        Trace traces[] = TraceData.fetchTraces(network, station, location, channel, startDate, endDate, 'B', includeResponse);
        
        for (Trace trace : traces) {
        	System.out.printf("Found %2s-%5s (%2s)  from %15s to %15s\n", trace.getNetwork(),trace.getStation(),trace.getChannel(), trace.getStartTime().toString(), trace.getEndTime().toString());
        	System.out.printf("  This trace has %d samples, at %7.2f samples per second\n", trace.getSampleCount(), trace.getSampleRate());
        	System.out.printf("  This channel is located at: %8.4f lat, %8.4f lon, elev %.0f m\n\n", trace.getLatitude(), trace.getLongitude(), trace.getElevation());
        }
        
        
	}
}
