package edu.iris.dmc.ws.examples;



import java.io.IOException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.List;
import java.util.Date;
import java.util.TimeZone;

import edu.iris.dmc.criteria.CriteriaException;
import edu.iris.dmc.criteria.WaveformCriteria;
import edu.iris.dmc.service.NoDataFoundException;
import edu.iris.dmc.service.ServiceNotSupportedException;
import edu.iris.dmc.service.ServiceUtil;
import edu.iris.dmc.service.WaveformService;
import edu.iris.dmc.timeseries.model.Segment;
import edu.iris.dmc.timeseries.model.Segment.Type;
import edu.iris.dmc.timeseries.model.Timeseries;

public class WaveformExample {

    public static void main(String args[]) {

        try {

            // initialize the service
            ServiceUtil serviceUtil = ServiceUtil.getInstance();
            serviceUtil.setAppName("Examples");
            WaveformService waveFormService = serviceUtil.getWaveformService();

            // define the time range of interest
            DateFormat dfm = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS");
            dfm.setTimeZone(TimeZone.getTimeZone("GMT"));
            Date startDate = dfm.parse("2006-09-11T00:00:00.000");
            Date endDate = dfm.parse("2006-09-11T00:01:45.000");

            //
            // Define the network, station, location and channels to retrieve.
            // 
            String network = "IU";
            String station = "ANMO";
            String location = "00";
            String channels[] = {"LHZ", "LH1", "LH2"};

            WaveformCriteria criteria = new WaveformCriteria();
            criteria.add(network, station, location, channels[0], startDate, endDate);
            criteria.add(network, station, location, channels[1], startDate, endDate);
            criteria.add(network, station, location, channels[2], startDate, endDate);

            // fetch the data
            // this will invoke http://www.iris.edu/ws/bulkdataselect
            List<Timeseries> timeseriesList = waveFormService.fetch(criteria);

            // process the fetched data.
            for (Timeseries timeseries : timeseriesList) {

                String net = timeseries.getNetworkCode();
                String sta = timeseries.getStationCode();
                String loc = timeseries.getLocation();
                String cha = timeseries.getChannelCode();


                System.out.printf("%s %s, %s, %s\n", net, sta, loc.replace(' ', '-'), cha);

                for (Segment segment : timeseries.getSegments()) {

                    int count = segment.getSampleCount();
                    String startTime = dfm.format(segment.getStartTime());
                    String endTime = dfm.format(segment.getEndTime());
                    float sampleRate = segment.getSamplerate();
                    System.out.printf("samples: %d, timerange: %s - %s, sps: %f\n", count, startTime, endTime, sampleRate);

                    if (segment.getType() == Type.INTEGER) {
                        List<Integer> data = segment.getIntData();
                        for (int d : data) {
                            System.out.printf("%d\n", d);
                        }

                    } else if (segment.getType() == Type.FLOAT) {
                        // handle float
                        List<Float> data = segment.getFloatData();
                        for (float d : data) {
                            System.out.printf("%f\n", d);
                        }
                    } else {
                        // deal with other data type here
                    }
                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        } catch (NoDataFoundException e) {
            e.printStackTrace();
        } catch (ParseException e) {
            e.printStackTrace();
        } catch (CriteriaException e) {
            e.printStackTrace();
        } catch (ServiceNotSupportedException e) {
			e.printStackTrace();
		}

    }

}
