package edu.iris.dmc.ws.tutorial;

//Import the error handling classes
import edu.iris.dmc.criteria.CriteriaException; // error handling
import edu.iris.dmc.service.NoDataFoundException; // error handling
import edu.iris.dmc.service.ServiceNotSupportedException;

//ServiceUtil is where we obtain all the "hooks" to the web services.
import edu.iris.dmc.service.ServiceUtil;

//Import the classes specific to the STATION example
import edu.iris.dmc.service.StationService; // the "hook" to ws-station
import edu.iris.dmc.criteria.OutputLevel; // used to specify level of detail
import edu.iris.dmc.criteria.StationCriteria; // used to dictate which data to fetch
import edu.iris.dmc.fdsn.station.model.Network; // Fetch returns a Network List
import edu.iris.dmc.fdsn.station.model.Station; // which contains stations...
import edu.iris.dmc.fdsn.station.model.Channel; // which contain channels...

//Import other Java classes
import java.util.List;
import java.util.Date;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.text.DateFormat;
import java.util.TimeZone;

//Here's the main class, where all the tutorial code will go
public class StationTutorial {

	public static void main(String[] args) throws NoDataFoundException,
			CriteriaException, ParseException, IOException,
			ServiceNotSupportedException {
		// get access to the services
		ServiceUtil serviceUtil = ServiceUtil.getInstance();
		serviceUtil.setAppName("Tutorials");
		StationService stationService = serviceUtil.getStationService();

		// specify criteria
		// define the dates of interest
		DateFormat dfm = new SimpleDateFormat("yyyy-MM-dd");
		dfm.setTimeZone(TimeZone.getTimeZone("GMT"));

		Date startDate = dfm.parse("1993-01-01");
		Date endDate = dfm.parse("2002-01-01");

		// specify the search criteria
		StationCriteria stationCriteria = new StationCriteria();
		stationCriteria.addNetwork("IU").addStation("ANMO");
		stationCriteria.setStartAfter(startDate).setEndBefore(endDate);

		// fetch data
		// fetch the data from the service, using the stationCriteria
		// The ws-station service can be queried for different levels of detail.
		// We're asking for CHANNEL detail, but could have chosen STATION or
		// NETWORK, too.

		List<Network> ls = stationService.fetch(stationCriteria,
				OutputLevel.CHANNEL);

		// process data
		// loop through all the channels to display the details
		for (Network n : ls) {
			System.out.printf("Network : %s with %d stations\n", n.getCode(),
					n.getSelectedNumberStations());
			// each network may have multiple stations
			for (Station s : n.getStations()) {
				System.out.printf("\tStation : %s with %d channels\n",
						s.getCode(), s.getChannels().size());
				for (Channel c : s.getChannels()) {
					// Finally! Print the detail
					System.out.printf("\t\tChannel: %s on: %s   off:%s\n",
							c.getCode(), c.getStartDate(), c.getEndDate());
				}
			}
		}
	}
}
