package edu.iris.dmc.ws.tutorial;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.TimeZone;

import edu.iris.dmc.criteria.CriteriaException;
import edu.iris.dmc.criteria.WaveformCriteria;
import edu.iris.dmc.service.NoDataFoundException;
import edu.iris.dmc.service.ServiceNotSupportedException;
import edu.iris.dmc.service.ServiceUtil;
import edu.iris.dmc.service.WaveformService;
import edu.iris.dmc.timeseries.model.Segment;
import edu.iris.dmc.timeseries.model.Timeseries;

public class WaveformTutorial {

	public static void main(String[] args) throws FileNotFoundException,
			IOException, ParseException, NoDataFoundException,
			CriteriaException, ServiceNotSupportedException {
		// We're specifying the BulkDataRequest equivalent of:
		// IU ANMO 00 BHZ 2006-09-11T00:00:00 2006-09-11T00:01:00

		// Initialize the service
		ServiceUtil serviceUtil = ServiceUtil.getInstance();
		serviceUtil.setAppName("Tutorials");
		WaveformService waveformService = serviceUtil.getWaveformService();

		// Specify the criteria
		// define the dates of interest
		DateFormat dfm = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
		dfm.setTimeZone(TimeZone.getTimeZone("GMT"));
		Date startDate = dfm.parse("2006-09-11T00:00:00");
		Date endDate = dfm.parse("2006-09-11T00:01:00");

		WaveformCriteria criteria = new WaveformCriteria();
		criteria.add("IU", "ANMO", "00", "BHZ", startDate, endDate);
		criteria.add("IU", "ANMO", "10", "BHZ", startDate, endDate);
		// to request additional data, we could use more criteria.add(...)
		// statements

		// fetch a list of Timeseries objects
		List<Timeseries> timeSeriesCollection = waveformService.fetch(criteria);

		// process the results
		for (Timeseries timeseries : timeSeriesCollection) {
			System.out.println(timeseries.getNetworkCode() + "-"
					+ timeseries.getStationCode() + " ("
					+ timeseries.getChannelCode() + "), loc:"
					+ timeseries.getLocation());
			for (Segment segment : timeseries.getSegments()) {
				System.out.printf("Segment:\n");
				System.out.printf("  Start: %s", segment.getStartTime());
				System.out.printf("  %d samples exist in this segment\n",
						segment.getSampleCount());

			}
		}

	}

}
