package edu.iris.dmc.service;

import static java.net.HttpURLConnection.HTTP_OK;
import static java.util.logging.Level.SEVERE;
import static java.util.logging.Level.WARNING;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.net.Authenticator;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.PasswordAuthentication;
import java.net.URL;
import java.net.URLStreamHandler;
import java.net.URLStreamHandlerFactory;
import java.util.logging.Level;
import java.util.logging.Logger;

import sun.net.www.protocol.http.Handler;

import edu.iris.dmc.criteria.Criteria;
import edu.iris.dmc.criteria.CriteriaException;
import edu.iris.dmc.ws.util.StringUtil;

public class BaseService {

	private Logger logger = Logger
			.getLogger("edu.iris.dmc.service.BaseService");

	public static int DEFAULT_READ_TIMEOUT_IN_MS = 180000;

	protected String appName = "";
	protected String baseUrl;

	private String version;
	private String serviceVersion;
	protected String userAgent;
	private boolean authenticate = false;

	protected String username;
	protected String password;

	public BaseService(String version, String v, String userAgent) {
		this.version = version;
		this.serviceVersion = v;
		this.userAgent = userAgent + "/" + this.version;

	}

	protected void authenticate(boolean authenticate) {
		this.authenticate = authenticate;
	}

	protected HttpURLConnection getConnection(URL u/*String baseUrl, String params*/)
			throws IOException, ServiceNotSupportedException {
		
		String host = u.getHost();
		int port = u.getPort();
		String path = u.getPath();
		if(path!=null && path.contains("query")){
			path=path.replace("query", "");
		}
		String baseUrl = "http://"+host;
		
		if(port>-1 && port !=80){
			baseUrl=baseUrl+":"+port;
		}
		baseUrl=baseUrl+path;
		

		Handler handler = new sun.net.www.protocol.http.Handler();
		URL url = new URL(new URL(baseUrl + "version"), baseUrl + "version",
				handler);
		HttpURLConnection connection = (HttpURLConnection) url.openConnection();
		String uAgent = this.userAgent;
		if (this.appName != null && !"".equals(this.appName)) {
			uAgent = uAgent + " (" + this.appName + ")";
		}
		connection.setRequestProperty("User-Agent", uAgent);

		connection.setRequestMethod("GET");
		connection.setRequestProperty("Accept", "text/plain");
		connection.connect();
		int responseCode = connection.getResponseCode();
		InputStream inputStream = responseCode != HTTP_OK ? connection
				.getErrorStream() : connection.getInputStream();

		if (responseCode != 200) {
			throw new IOException(connection.getResponseMessage());
		}

		String versionText = StringUtil.toString(inputStream);

		String[] vComponents = versionText.split("\\.");

		String[] lvComponents = this.serviceVersion.split("\\.");

		try {
			if (!vComponents[0].equals(lvComponents[0])
					|| !vComponents[1].equals(lvComponents[1])) {
				throw new ServiceNotSupportedException(
						"Version is not supported.");
			}
		} finally {
			connection.disconnect();
		}

		/*String u = baseUrl;
		if (authenticate) {
			u = u + "queryauth";
			authenticate();
		} else {
			u = u + "query";
		}

		if (params != null) {
			u = u + "?" + params;
		}*/

		url = new URL(u, u.toString(), handler);

		// url = new URL(u);

		connection = (HttpURLConnection) url.openConnection();

		connection.setRequestProperty("User-Agent", uAgent);
		connection.setReadTimeout(DEFAULT_READ_TIMEOUT_IN_MS);
		return connection;
	}

	
	private void authenticate() {
		/**
		 * We do this to bypass MatLab limitations
		 */
		sun.net.www.protocol.http.AuthCacheValue
				.setAuthCache(new sun.net.www.protocol.http.AuthCacheImpl());
		Authenticator.setDefault(new Authenticator() {
			@Override
			protected PasswordAuthentication getPasswordAuthentication() {
				return new PasswordAuthentication(username, password
						.toCharArray());
			}
		});
	}

	/**
	 * Stream result into output stream, can be used to save result to a local
	 * file.
	 * 
	 * @param out
	 * @param criteria
	 * @throws NoDataFoundException
	 *             (404)
	 * @throws CriteriaException
	 *             (400)
	 * @throws IOException
	 * @throws ServiceNotSupportedException
	 * @throws UnauthorizedAccessException
	 */
	public void stream(OutputStream out, Criteria criteria)
			throws NoDataFoundException, CriteriaException, IOException,
			ServiceNotSupportedException, UnauthorizedAccessException {

		if (logger.isLoggable(Level.FINER)) {
			logger.entering(this.getClass().getName(),
					"find(OutputStream out, Criteria criteria)",
					new Object[] { criteria });
		}

		
		URL theQuery = null;
		try {
			theQuery = new URL(this.baseUrl+"query?"+criteria.toUrlParams());
			
		} catch (MalformedURLException e) {
			throw new IOException("Invalid url: " + this.baseUrl+"query?"+criteria.toUrlParams());
		}
		
		String paramsString = null;
		try {
			paramsString = criteria.toUrlParams().get(0);
		} catch (UnsupportedEncodingException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}

		HttpURLConnection connection = null;

		try {

			connection = getConnection(theQuery);
			connection.setRequestProperty("Content-Type",
					"application/x-www-form-urlencoded");
			connection.setDoOutput(true);
			connection.connect();

			OutputStream outputStream = connection.getOutputStream();
			OutputStreamWriter writer = new OutputStreamWriter(outputStream);

			writer.write(paramsString);
			writer.close();
			outputStream.close();

			int responseCode = connection.getResponseCode();
			InputStream inputStream = responseCode != HTTP_OK ? connection
					.getErrorStream() : connection.getInputStream();

			switch (responseCode) {
			case 404:
				throw new NoDataFoundException("No data found for: " + criteria);
			case 401:
				throw new UnauthorizedAccessException("Unauthorized access: "
						+ criteria);
			case 400:
				throw new CriteriaException("Bad request parameter: "
						+ StringUtil.toString(inputStream));
			case 500:
				if (logger.isLoggable(WARNING))
					logger.warning("An error occurred while making a GET request "
							+ StringUtil.toString(inputStream));
				throw new IOException(StringUtil.toString(inputStream));
			case 200:
				BufferedInputStream data = new BufferedInputStream(inputStream);

				BufferedOutputStream bout = new BufferedOutputStream(out);

				while (true) {
					int datum = data.read();
					if (datum == -1)
						break;
					bout.write(datum);
				}
				bout.flush();

				if (logger.isLoggable(Level.INFO)) {
					logger.info("Finished writing result to outputstream.");
				}

				if (logger.isLoggable(Level.FINER)) {
					// Use the following if the method does not return a value
					logger.exiting(this.getClass().getName(),
							"find(OutputStream out, Criteria criteria)");
				}
				break;
			default:
				if (logger.isLoggable(WARNING))
					logger.warning("An error occurred while making a GET request "
							+ StringUtil.toString(inputStream)
							+ ". code:"
							+ responseCode);
				throw new IOException(StringUtil.toString(inputStream));
			}

		} finally {
			if (connection != null) {
				connection.disconnect();
			}
			if (out != null) {
				try {
					out.flush();
					out.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}

	}

	/**
	 * Provide an appName included in the User-Agent. This text string is
	 * intended to uniquely identify an application. Preferred string format:
	 * "APPNAME/VERSION"
	 * 
	 * @param appName
	 */
	public void setAppName(String appName) {
		this.appName = appName;
	}

	/**
	 * 
	 * @return the App name used for User-Agent
	 */
	public String getAppName() {
		return this.appName;
	}

	public String getBaseUrl() {
		return this.baseUrl;
	}

	public void setBaseUrl(String url) {
		this.baseUrl = url;
	}
}
