package edu.iris.dmc.service;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigInteger;
import java.net.HttpURLConnection;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Iterator;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.stream.EventFilter;
import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.events.Attribute;
import javax.xml.stream.events.StartElement;
import javax.xml.stream.events.XMLEvent;

import edu.iris.dmc.fdsn.station.model.Network;
import edu.iris.dmc.fdsn.station.model.RestrictedStatusType;
import edu.iris.dmc.fdsn.station.model.Station;

public class StationIterator implements Iterator<Station> {

	private HttpURLConnection connection;
	private InputStream inputStream;
	private XMLEventReader xmlEventFilteredReader;
	private XMLEventReader xmlEventReader;
	private Unmarshaller unmarshaller;

	public StationIterator(HttpURLConnection connection, InputStream inputStream)
			throws FileNotFoundException, XMLStreamException, JAXBException {
		this(inputStream);
		this.connection = connection;
	}

	public StationIterator(InputStream inputStream)
			throws FileNotFoundException, XMLStreamException, JAXBException {
		this.inputStream = inputStream;

		XMLInputFactory xmlFactory = XMLInputFactory.newInstance();

		xmlEventReader = xmlFactory.createXMLEventReader(this.inputStream);

		EventFilter filter = new EventFilter() {
			public boolean accept(XMLEvent event) {
				return event.isStartElement();
			}
		};

		JAXBContext jc = JAXBContext
				.newInstance(edu.iris.dmc.fdsn.station.model.ObjectFactory.class);
		unmarshaller = jc.createUnmarshaller();

		xmlEventFilteredReader = xmlFactory.createFilteredReader(
				xmlEventReader, filter);

		/*StartElement se = (StartElement) xmlEventFilteredReader.nextEvent();

		if (!"Network".equals(se.getName().getLocalPart())) {
			// TODO: throw exception
		}*/

	}

	private Network network;
	@Override
	public boolean hasNext() {
		XMLEvent e = null;
		try {
			StartElement se = null;
			while ((e = xmlEventFilteredReader.peek()) != null) {

				if (e.isStartElement()) {
					se = (StartElement) e;

					if ((((StartElement) e)).getName().getLocalPart()
							.equals("Station")) {
						return true;
					} else if ((((StartElement) e)).getName().getLocalPart()
							.equals("Network")) {
						network = new Network();

						Iterator<Attribute> attributes = se.getAttributes();
						while (attributes.hasNext()) {
							Attribute attribute = attributes.next();
							String attributeName = attribute.getName()
									.getLocalPart();
							if (attributeName.equals("code")) {
								network.setCode(attribute.getValue());
							} else if (attributeName.equals("startDate")) {
								String startString = attribute.getValue();
								if (startString != null) {
									SimpleDateFormat sdf = new SimpleDateFormat(
											"yyyy-MM-dd'T'HH:mm:ss");
									try {
										Date date = sdf.parse(startString);
										GregorianCalendar c = new GregorianCalendar();
										c.setTime(date);
										XMLGregorianCalendar cal2 = DatatypeFactory
												.newInstance()
												.newXMLGregorianCalendar(c);
										network.setStartDate(cal2);
									} catch (ParseException e1) {
										// TODO Auto-generated catch block
										e1.printStackTrace();
									} catch (DatatypeConfigurationException e2) {
										// TODO Auto-generated catch block
										e2.printStackTrace();
									}
								}
							} else if (attributeName.equals("endDate")) {
								String endString = attribute.getValue();
								if (endString != null) {
									SimpleDateFormat sdf = new SimpleDateFormat(
											"yyyy-MM-dd'T'HH:mm:ss");
									try {
										Date date = sdf.parse(endString);
										GregorianCalendar c = new GregorianCalendar();
										c.setTime(date);
										XMLGregorianCalendar cal2 = DatatypeFactory
												.newInstance()
												.newXMLGregorianCalendar(c);
										network.setEndDate(cal2);
									} catch (ParseException e1) {
										// TODO Auto-generated catch block
										e1.printStackTrace();
									} catch (DatatypeConfigurationException e2) {
										// TODO Auto-generated catch block
										e2.printStackTrace();
									}
								}
							} else if (attributeName.equals("restrictedStatus")) {
								network.setRestrictedStatus(RestrictedStatusType
										.fromValue(attribute.getValue()));
							} else {
								// Do nothing for now
							}
						}

						xmlEventFilteredReader.next();
					} else if (((StartElement) e).getName().getLocalPart()
							.equals("Description")) {
						XMLEvent event = xmlEventFilteredReader.nextEvent();
						if (event.isCharacters()) {
							network.setDescription(event.asCharacters()
									.getData());
						}
					} else if (((StartElement) e).getName().getLocalPart()
							.equals("TotalNumberStations")) {
						XMLEvent event = xmlEventFilteredReader.nextEvent();
						if (event.isCharacters()) {
							String data = event.asCharacters().getData();
							if (data != null) {
								network.setTotalNumberStations(new BigInteger(
										data));
							}
						}
					}else {
						xmlEventFilteredReader.next();
					}
				} else {
					xmlEventFilteredReader.next();
				}
			}
			return false;
		} catch (XMLStreamException se) {
			close();
			throw new IllegalStateException(se.toString());
		}
	}

	@Override
	public Station next() {
		try {
			Object o = unmarshaller.unmarshal(xmlEventReader,
					edu.iris.dmc.fdsn.station.model.Station.class);

			if (o instanceof javax.xml.bind.JAXBElement) {
				Object jo = ((JAXBElement) o).getValue();
				if (jo instanceof edu.iris.dmc.fdsn.station.model.Station) {
					edu.iris.dmc.fdsn.station.model.Station station = (edu.iris.dmc.fdsn.station.model.Station) jo;
					station.setNetwork(network);
					return station;
				}
			}
			throw new IllegalStateException("Unknown state");
		} catch (JAXBException e) {
			throw new IllegalStateException(e.toString());
		}
	}

	@Override
	public void remove() {
		throw new UnsupportedOperationException(
				"StationIterator.remove() is not supported");
	}

	public void close() {

		if (this.connection != null) {
			this.connection.disconnect();
			this.connection = null;
		}

		try {
			if (this.inputStream != null) {
				this.inputStream.close();
				this.inputStream = null;
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

}
